import json
import requests

mcURL = ""
mcPort = -1
mcToken = ""
mcVerifySSL = True

def initializeMCParams(url, port, token, verifySSL):
    global mcURL
    global mcPort
    global mcToken
    global mcVerifySSL
    mcURL = url
    mcPort = port
    mcToken = "Token " + token
    mcVerifySSL = verifySSL 

def getAPIRequest(APIReq) -> json:
    URL = mcURL + ":" + str(mcPort) + APIReq
    headersData = {
        "Authorization": mcToken
    }
    req = requests.get(URL, headers=headersData, verify=mcVerifySSL)
    return json.loads(req.content)

def postAPIRequest(APIReq, bodyData) -> json:
    URL = mcURL + ":" + str(mcPort) + APIReq
    bodyData = json.dumps(bodyData)
    headersData = {
        "Authorization": mcToken,
        "Content-Type": "application/json",
        "Content-Length": str(len(bodyData))
    }
    req = requests.post(URL, headers=headersData, data=bodyData, verify=mcVerifySSL)
    return json.loads(req.content)

def deleteAPIRequest(APIReq) -> bool:
    URL = mcURL + ":" + str(mcPort) + APIReq
    headersData = {
        "Authorization": mcToken,
        "Content-Type": "application/json"
    }
    req = requests.delete(URL, headers=headersData, verify=mcVerifySSL)
    return (req.status_code == 204)



import sys
import json
import logging

sys.path.append("./")
from ..communication.communication import getAPIRequest  

def getAgentList():
    return getAPIRequest("/api/v2/agents")

def getAgentByAttrs(attr1Name, attr1Value, attr2Name = None, attr2Value = None):
    agents = getAgentList()
    if (attr1Value == "*"):
        # TO DO: properly handle regex
        obj = [obj for obj in agents if (obj[attr2Name]==attr2Value)]
    else:    
        obj = [obj for obj in agents if ((obj[attr1Name]==attr1Value) and ((attr2Name==None) or (obj[attr2Name]==attr2Value)))]
    if (len(obj) == 0):
        print("failed to find Agent where '" + attr1Name + "'='" + attr1Value + "' and '" + attr2Name + "'='" + attr2Value + "'")
        logging.error("failed to find Agent where '" + attr1Name + "'='" + attr1Value + "' and '" + attr2Name + "'='" + attr2Value + "'")
    return obj



import csv
import time
import threading
import logging
import os

from ..jobs.jobs import addSimpleSyncJob, getJobs, getJobsByAttrs, getJobRunID, getJobByID

class jobsFromCSV:
    def __init__(self, csvFileName, jobsMonitor, callbackFunction, concurrentJobsCount, monitorInterval):
        self.csvFileName = csvFileName
        self.jobsList = []
        self.jobDict = {}
        self.nextJob = 0
        self.jobsMonitor = jobsMonitor
        self.callbackFunction = callbackFunction
        self.concurrentJobsCount = concurrentJobsCount
        self.monitorInterval = monitorInterval

        # read the csv file
        self.readCSV()
        # check the current state of the system
        self.syncCurrentState()
        # start the job creation and monitoring process
        threading.Thread(target=self.jobCreationThread).start()

    def readCSV(self):
        try:
            with open(self.csvFileName, "r") as file:
                csvReader = csv.DictReader(file, skipinitialspace=True)
                i = 0
                for row in csvReader:
                    self.jobsList.append(row)
                    self.jobDict[row['jobName']] = i
                    i += 1
                print(self.jobsList)
        except:
            print("failed to read the csv file: '" + self.csvFileName + "'")
            os._exit(1)

    def updateCSV(self):
        with open(self.csvFileName, 'w') as csvfile:  
            writer = csv.DictWriter(csvfile, self.jobsList[0].keys())
            writer.writeheader()
            writer.writerows(self.jobsList)  


    def addNextJob(self):
        if (self.nextJob < len(self.jobsList)):
            try:
                addSimpleSyncJob(
                    self.jobsList[self.nextJob]['jobName'],
                    self.jobsList[self.nextJob]['jobDescription'],
                    self.jobCompletedCallback,
                    self.jobsList[self.nextJob]['ip1'],
                    self.jobsList[self.nextJob]['name1'],
                    self.jobsList[self.nextJob]['permission1'],
                    self.jobsList[self.nextJob]['folder1'],
                    self.jobsList[self.nextJob]['ip2'],
                    self.jobsList[self.nextJob]['name2'],
                    self.jobsList[self.nextJob]['permission2'],
                    self.jobsList[self.nextJob]['folder2'],
                    self.jobsMonitor
                    )
            except:
                print("failed to add job " + self.jobsList[self.nextJob])
                logging.error("failed to add job " + self.jobsList[self.nextJob])
            self.nextJob += 1

    def handleExistingJob(self, jobExists):
        print("Job: '" + self.jobsList[self.nextJob]['jobName'] + "' was already running")
        logging.info("Job: '" + self.jobsList[self.nextJob]['jobName'] + "' was already running")
        jobRunID = getJobRunID(jobExists[0]['id'])
        self.jobsMonitor.addMonitoredJob(jobRunID['data'][0]['id'], self.jobCompletedCallback)

    def syncCurrentState(self):
        for index in range(len(self.jobsList)):
            jobExists = getJobsByAttrs("name", self.jobsList[index]['jobName'])
            if (jobExists):
                self.handleExistingJob(jobExists)

    def jobCompletedCallback(self, jobID, runID):
        # persist completion state
        finishedJob = getJobByID(jobID)
        self.jobsList[self.jobDict[finishedJob['name']]]['completed'] = 'y'
        self.updateCSV()
        self.callbackFunction(jobID, runID)

    def jobCreationThread(self):
        while (True):
            jobs = getJobs()
            while ((self.nextJob < len(self.jobsList)) and (len(jobs) < self.concurrentJobsCount)):
                jobExists = getJobsByAttrs("name", self.jobsList[self.nextJob]['jobName'])
                if self.jobsList[self.nextJob]['completed'] == 'y':
                    self.nextJob += 1
                elif (not jobExists):
                    self.addNextJob()
                    jobs = getJobs()
                else:
                    self.handleExistingJob(jobExists)
                    self.nextJob += 1
            time.sleep(self.monitorInterval)



import sys
import os
import json
import os
import logging
import getopt
from signal import signal, SIGINT


#sys.path.append("./") 
from .communication.communication import initializeMCParams, getAPIRequest
from .agents.agents import getAgentList, getAgentByAttrs
from .jobs.jobs import appendToJobAgentList, addJob, getJobRunID, startJob, deleteJob, jobsMonitor, addSimpleSyncJob, getJobRunStatus, getJobByID
from .jobs_from_csv.jobs_from_csv import jobsFromCSV
 
# default values
csvFileName = "./jobs.csv"
mcURL = os.getenv('RESILIO_MC_URL')
mcAuth = os.getenv('RESILIO_AUTH_TOKEN')
jobCount = 4 
logFile = "./pymc.log" 
verifySSL = True

# ctrl-c handler
def handler(signal_received, frame):
    # Handle any cleanup here
    print('SIGINT or CTRL-C detected. Exiting gracefully')
    os._exit(1)

def doSomethingWhenJobIsDone(jobID, runID):
    print("function called after the Job " + str(jobID) + " was done")
    logging.info("Job " + str(jobID) + " was completed and will be deleted")
    jobDetails = getJobByID(jobID)
    logging.info(json.dumps(jobDetails))
    runResult = getJobRunStatus(runID)
    logging.info(json.dumps(runResult))
    deleteJob(jobID)

# TO DO: change this to take command line arguments for csv file name, how many jobs, mc url, auth token
def getArgs():
    global csvFileName
    global mcURL
    global mcAuth
    global jobCount
    global logFile
    global verifySSL
    argumentList = sys.argv[1:]
    options = "c:j:m:a:o:nh"
    long_options = ["csvfile", "jobcount", "mcaddress", "authtoken", "output", "help", "nosslverification"]

    try:
        arguments, values = getopt.getopt(argumentList, options, long_options)
        for currentArgument, currentValue in arguments:
            if currentArgument in ("-c", "--csvfile"):
                csvFileName = currentValue
            elif currentArgument in ("-j", "--jobcount"):
                jobCount = int(currentValue)
            elif currentArgument in ("-m", "--mcaddress"):
                mcURL = currentValue
            elif currentArgument in ("-a", "--authtoken"):
                mcAuth = currentValue
            elif currentArgument in ("-o", "--output"):
                logFile = currentValue
            elif currentArgument in ("-n", "--nosslverification"):
                verifySSL = False
                import urllib3
                urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
            elif currentArgument in ("-h", "--help"):
                print ("Diplaying Help")
    except getopt.error as err:
        # output error, and return with an error code
        sys.exit(str(err))

def main():
    # exit on ctrl-c
    signal(SIGINT, handler)

    # read the command line args
    getArgs()

    # configure a log file
    logging.basicConfig(filename=logFile, filemode='a', level=logging.INFO)

    # initialize MC
    initializeMCParams(mcURL, 8443, mcAuth, verifySSL)

    # quick test that we can actually connect to this Management Console
    print(getAPIRequest("/api/v2/info"))

    # start the job monitor
    myJobsMonitor = jobsMonitor(10)

    # get the list of agents
    getAgentList()

    # start jobs based on a csv
    jobsFromCSV(csvFileName, myJobsMonitor, doSomethingWhenJobIsDone, jobCount, 10)

main()


import sys
import json
import time
import threading
import logging

#sys.path.append("./")
from ..communication.communication import getAPIRequest, postAPIRequest, deleteAPIRequest
from ..agents.agents import getAgentByAttrs

def appendToJobAgentList(list, id, permission, path) -> json: 
    list.append({
        "id": id,
        "permission": permission,
        "path": {"linux": path, "win": path, "osx": path, "android": path, "xbox": path }
    })
    return list

def addJob(name, desc, type, agents) -> json:
    jobInfo = {
        "name": name,
        "description": desc,
        "type": type,
        "agents": agents
    }
    return postAPIRequest("/api/v2/jobs", jobInfo)

def startJob(jobID) -> json:
    jobInfo = {
        "job_id": jobID
    }
    return postAPIRequest("/api/v2/runs", jobInfo)

def getJobs() -> json:
    return getAPIRequest("/api/v2/jobs")

def getJobByID(jobID) -> json:
    return getAPIRequest("/api/v2/jobs/" + str(jobID))

def getJobsByAttrs(attr1Name, attr1Value, attr2Name = None, attr2Value = None):
    jobs = getAPIRequest("/api/v2/jobs")
    return [obj for obj in jobs if ((obj[attr1Name]==attr1Value) and ((attr2Name==None) or (obj[attr2Name]==attr2Value)))]

def getJobRunID(jobID) -> json:
    return getAPIRequest("/api/v2/runs?job_id=" + str(jobID))

def getJobRunStatus(runID) -> json:
    return getAPIRequest("/api/v2/runs/" + str(runID))

def deleteJob(jobID) -> json:
    return deleteAPIRequest("/api/v2/jobs/" + str(jobID))

def addSimpleSyncJob(jobName, jobDescription, callbackfunction,
                    agent1IP, agent1Name, agent1Permission, agent1Folder,
                    agent2IP, agent2Name, agent2Permission, agent2Folder,
                    myJobsMonitor):
    try:
        jobAgentList = []
        jobAgentList = appendToJobAgentList(jobAgentList, getAgentByAttrs("ip", agent1IP, "name", agent1Name)[0]['id'], agent1Permission, agent1Folder)   
        jobAgentList = appendToJobAgentList(jobAgentList, getAgentByAttrs("ip", agent2IP, "name", agent2Name)[0]['id'], agent2Permission, agent2Folder)   
        newJob = addJob(jobName, jobDescription, "sync", jobAgentList)
        newJobRunID = getJobRunID(newJob['id'])
        myJobsMonitor.addMonitoredJob(newJobRunID['data'][0]['id'], callbackfunction)
    except:
        print("failed to add job '" + jobName + "'")
        logging.error("failed to add job '" + jobName + "'")

class jobRunMonitor:
    def __init__(self, runID, finishedCallbackFunction):
        self.monitorJobID = 0
        self.monitorRunID = runID
        self.monitorJobStatus = ""
        self.monitorErrCode = 200
        self.monitorCallback = finishedCallbackFunction

    def getJobStatus(self) -> str:
        return self.monitorJobStatus
    
    def getErrCode(self) -> int:
        return self.monitorErrCode

    def updateJobRunStatus(self):
        runStatus = getJobRunStatus(self.monitorRunID)
        self.monitorJobID = runStatus["job_id"]
        self.monitorJobStatus = runStatus["status"]
        try:
            self.monitorErrCode = runStatus["code"]
        except:
            self.monitorErrCode = 0
        if (self.monitorJobStatus == "finished"):
            self.monitorCallback(self.monitorJobID, self.monitorRunID)

class jobsMonitor:
    def __init__(self, monitorInterval):
        self.monitorInterval = monitorInterval
        self.jobRuns = {}
        self.jobRunsLock = threading.Lock()
        threading.Thread(target=self.jobMonitorThread).start()  # , daemon=True

    def addMonitoredJob(self, runID, finishedCallbackFunction):
        with self.jobRunsLock:
            if (not runID in self.jobRuns):
                # new job run to monitor
                self.jobRuns[runID] = jobRunMonitor(runID, finishedCallbackFunction)

    def jobMonitorThread(self):
        while (True):
            with self.jobRunsLock:
                ids = list(self.jobRuns.keys())
                for runID in ids:
                    if ((self.jobRuns[runID].getJobStatus() != "finished") and (self.jobRuns[runID].getErrCode != 404)):        
                        self.jobRuns[runID].updateJobRunStatus()
                    else: 
                        self.jobRuns.pop(runID, None)
            time.sleep(self.monitorInterval)
        



import setuptools 

setuptools.setup(name='py-resilio-connect',
      version='1.3.1',
      description='Generate simple Resilio Connect Sync Jobs based on a csv file',
      url='https://github.com/ilanshamir/connect-scripts.git',
      author='Ilan Shamir',
      author_email='team@resilio.com',
      license='MIT',
      packages=setuptools.find_packages(),
      install_requires=[
         "requests",
      ],
      zip_safe=False)